/**************************************************************************/
/*                                                                        */
/*  Thibaut Balabonski, Sylvain Conchon, Jean-Christophe Filliâtre,       */
/*  Kim Nguyen, Laurent Sartre                                            */
/*                                                                        */
/*  Informatique - MP2I/MPI - CPGE 1re et 2e années.                      */
/*  Cours et exercices corrigés. Éditions Ellipses, 2022.                 */
/*                                                                        */
/*  https://www.informatique-mpi.fr/                                      */
/*                                                                        */
/**************************************************************************/

// structure union-find, avec union pondérée et compression de chemins

#include "prelude.h"

#include "union_find.h"

struct UnionFind {
  int size;
  int *link;
  int *rank;
};

uf *uf_create(int size) {
  assert(size > 0);
  uf *uf = malloc(sizeof(struct UnionFind));
  uf->size = size;
  uf->link = calloc(size, sizeof(int));
  for (int i = 0; i < size; i++)
    uf->link[i] = i;
  uf->rank = calloc(size, sizeof(int));
  return uf;
}

int uf_find(uf *uf, int i) {
  assert(0 <= i && i < uf->size);
  int p = uf->link[i];
  if (p == i) return i;
  int r = uf_find(uf, p);
  uf->link[i] = r;
  return r;
}

void uf_union(uf *uf, int i, int j) {
  assert(0 <= i && i < uf->size);
  assert(0 <= j && j < uf->size);
  int ri = uf_find(uf, i);
  int rj = uf_find(uf, j);
  if (ri == rj) return;
  if (uf->rank[ri] < uf->rank[rj])
    uf->link[ri] = rj;
  else {
    uf->link[rj] = ri;
    if (uf->rank[ri] == uf->rank[rj])
      uf->rank[ri]++;
  }
}

void uf_delete(uf *uf) {
  free(uf->link);
  free(uf->rank);
  free(uf);
}
